# Structure:
# main.py
# |- audio.py    - sfx, music
# |- controls.py - das, arr
# |- draw.py     - ui, stdout, terminal
# |- srs.py      - wallkicks and rotation
# |
# |- game.py     - scoring, logic, hold, etc
# |- pieces.py   - define tetrominoes
# |- config.py   - rebind controls, gravity, etc
# |
# \- /assets     - for audio.py (simpleaudio)
#    |- /sfx
#    \- /music

pass